import re
from itertools import combinations

punctuations = r"[¥•＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､　、〃〈〉《》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏﹑﹔·'℃°•·．﹑︰〈〉─《﹖﹣﹂﹁﹔！？｡。＂＃＄％＆＇（）＊＋，﹐－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.．!\"#$%&()*+,\-.\:;<=>?@\[\]\\\/^_`{|}~]"
httplink = r"(https|http)[:\/0-9a-zA-Z.?=&;]*"
unused = r":.+:|<br>|\\n|\(\w+\)|\[.+\]"


def clean_all(string):
    return clean_unused_tag(clean_httplink(clean_htmlelement(string)))


def clean_unused_tag(string):
    p = re.compile(unused)
    return p.sub(' ', string.strip()).strip()


def clean_htmlelement(data):
    p = re.compile(r'<.*?>')
    return p.sub(' ', data.strip()).strip()


def clean_httplink(string):
    p = re.compile(httplink)
    return p.sub(' ', string.strip()).strip()


def split_lines_by_punc(lines, max_len=50):
    sentences = []
    for line in lines:
        line = full2half(line.strip())
        for sentence in filter(None, re.split(punctuations, line)):
            sentence = sentence.strip()
            if len(sentence) > 0 and len(split_sentence_to_array(sentence)) < max_len:
                sentences.append(sentence)
    return sentences


def split_sentence_to_ngram(sentence, max_len=15):
    ngrams = []
    regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]"
    path = re.findall(regex, sentence, re.UNICODE)
    for i in range(len(path)):
        for j in range(1, min(len(path) + 1, max_len)):
            if i + j <= len(path):
                ngrams.append(join_words_to_sentence([subPath for subPath in path[i:i + j]]))
    return ngrams


def split_sentence_to_ngram_in_part(sentence, max_len=15):
    ngrams = []
    regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]"
    path = re.findall(regex, sentence, re.UNICODE)
    for i in range(len(path)):
        part = []
        for j in range(1, min(len(path) + 1, max_len)):
            if i + j <= len(path):
                part.append(join_words_to_sentence([subPath for subPath in path[i:i + j]]))

        ngrams.append(part)
    return ngrams


def split_text_in_all_comb(sentence):
    result = []
    for i in range(len(sentence)):
        ipart = "".join(sentence[:i + 1])
        irest = sentence[i + 1:]
        if len(irest) > len(ipart):
            for j in split_text_in_all_comb(irest):
                result.append((ipart + " " + j).rstrip())
        else:
            result.append((ipart + " " + join_words_to_sentence(irest)).rstrip())
    return result


def split_sentence_to_array(sentence, merge_non_eng=False):
    sentence = "".join(
        (char if char.isalpha() or char == " " else " " + char + " ") for char in sentence)  # separate punctuation
    if merge_non_eng:
        regex = r"[0-9]|[a-zA-Z]+\'*[a-z]*|[\w]+|[^ ]"
    else:
        regex = r"[0-9]|[a-zA-Z]+\'*[a-z]*|[\w]|[^ ]"
    path = re.findall(regex, sentence, re.UNICODE)
    return path


def join_words_to_sentence(array):
    return ''.join(
        [(' ' if i > 0 and is_all_english(str(subPath)) and not is_all_english(str(array[i - 1])) else '') + subPath + (
            ' ' if is_all_english(str(subPath)) else '') for i, subPath in enumerate(array)]).strip()


def passage_into_chunk(text, length):
    lines = (i.strip() for i in text.splitlines())
    result = []
    chunk = ''
    for line in lines:
        chunk += (line + " ")
        if len(chunk) > length:
            result.append(chunk)
            chunk = ''
    if len(chunk) > 0:
        result.append(chunk)
    return result


def is_all_english(s):
    try:
        s.encode(encoding='utf-8').decode('ascii')
    except:
        return False
    else:
        return True


def is_contain_number(s):
    return bool(re.search(r'[0-9]+', s))


def is_contain_english(s):
    return bool(re.search(r'[a-zA-Z]+', s))


def is_list_contain_string(f_str, f_list):
    return any(s in f_str for s in f_list) or any(f_str in s for s in f_list)


def full2half(s):
    n = []
    for char in s:
        num = ord(char)
        if num == 0x3000:
            num = 32
        elif 0xFF01 <= num <= 0xFF5E:
            num -= 0xfee0
        num = chr(num)
        n.append(num)
    return ''.join(n)


def half2full(s):
    n = []
    for char in s:
        num = ord(char)
        if num == 320:
            num = 0x3000
        elif 0x21 <= num <= 0x7E:
            num += 0xfee0
        num = chr(num)
        n.append(num)
    return ''.join(n)


def sliding_windows(seq, slide=128, append_seq=[]):
    windows_list = []
    slide_pos = []
    seq_length = len(seq)
    windows_min_size = min(slide, seq_length)
    for i in range(int(seq_length / slide) + 1):
        for end in [min(i * slide + slide, seq_length),
                    min(i * slide + slide - int(slide / 2), seq_length)]:
            start = max(0, end - slide)
            slide_seq = seq[start:end] + append_seq
            if len(
                    slide_seq) >= windows_min_size and slide_seq not in windows_list:  # avoid adding same list repeatedly
                windows_list.append(slide_seq)
                slide_pos.append([start, end])
    return windows_list, slide_pos


def jaccard_similarity(list1, list2):
    s1 = set(list1)
    s2 = set(list2)
    return len(s1.intersection(s2)) / len(s1.union(s2))


def is_jaccard_similar(s, t, threshold=0.5):
    return jaccard_similarity(s, t) > threshold


def filter_jaccard_similar_text_from_list(lst, max_return, threshold=0.5):
    while True:
        filtered = False
        for s, t in combinations(lst, 2):
            if is_jaccard_similar(s[0], t[0], threshold) and len(lst) - 1 >= max_return:
                lst.remove(t)
                filtered = True
                break
        if not filtered:
            break
    return lst



import unittest
from nlp2.text import *


class TestText(unittest.TestCase):

    def test_clean_all(self):
        self.assertEqual("無聊得過此帖？！    \n \n \n認同。 \n \n改洋名，只是一個字號。", clean_all(
            "<br><br>http://news.IN1802020028.htm[quote]原帖由 [i]234282[/i] 於 2019-1-18 06:46 PM 發表 [url=https://www.discuss.com.hk/redirect.php?goto=findpost&amp;pid=493614969&amp;ptid=27981082][img]https://www.discuss.com.hk/images/common/back.gif[/img][/url]<br>\n無聊得過此帖？！:smile_42: [/quote]<br>\n<br>\n<br>\n認同。<br>\n<br>\n改洋名，只是一個字號。"))
        self.assertEqual("Phraseg - 一言：新詞發現工具包 今天天氣 晴朗", clean_all(
            "[i]234282[/i] <div class=""><p>Phraseg - 一言：新詞發現工具包http://news.IN1802020028.htm今天天氣http://news.we028.晴朗</p></div>"))

    def test_clean_unused_tag(self):
        self.assertEqual(clean_unused_tag(
            "[quote]原帖由 [i]234282[/i] 於 2019-1-18 06:46 PM 發表 [url=https://www.discuss.com.hk/redirect.php?goto=findpost&amp;pid=493614969&amp;ptid=27981082][img]https://www.discuss.com.hk/images/common/back.gif[/img][/url]<br>\n無聊得過此帖？！:smile_42: [/quote]<br>\n<br>\n<br>\n認同。<br>\n<br>\n改洋名，只是一個字號。"),
            "無聊得過此帖？！    \n \n \n認同。 \n \n改洋名，只是一個字號。")
        self.assertEqual(clean_unused_tag(
            "[quote]<br>\n無聊得過此帖？！:smile_42: [/quote]<br>\n<br>\n<br>\n認同。<br>\n<br>\n改洋名，只是一個字號。"),
            "無聊得過此帖？！    \n \n \n認同。 \n \n改洋名，只是一個字號。")

    def test_clean_htmlelement(self):
        self.assertEqual(clean_htmlelement("<div class=""><p>Phraseg - 一言：新詞發現工具包</p></div>"), "Phraseg - 一言：新詞發現工具包")

    def test_clean_httplink(self):
        self.assertEqual(clean_httplink("http://news.IN1802020028.htm 今天天氣http://news.we028.晴朗"), "今天天氣 晴朗")
        self.assertEqual(clean_httplink(" https://forum.gamer.com.tw/C.php?bsn=60076&amp;snA=2817750  "), "")

    def test_split_lines_by_punc(self):
        self.assertEqual(split_lines_by_punc(["你好啊.hello，me"]), ['你好啊', 'hello', 'me'])

    def test_split_sentence_to_ngram(self):
        self.assertEqual(split_sentence_to_ngram("加州旅館"),
                         ['加', '加州', "加州旅", "加州旅館", "州", "州旅", "州旅館", "旅", "旅館", "館"])

    def test_split_sentence_to_ngram_in_part(self):
        self.assertEqual(split_sentence_to_ngram_in_part("加州旅館"),
                         [['加', '加州', "加州旅", "加州旅館"], ["州", "州旅", "州旅館"], ["旅", "旅館"], ["館"]])

    def test_split_text_in_all_ways(self):
        self.assertEqual(split_text_in_all_comb("加州旅館"),
                         ['加 州 旅 館', '加 州 旅館', '加 州旅 館', '加 州旅館', '加州 旅館', '加州旅 館', '加州旅館'])

    def test_split_sentence_to_array(self):
        self.assertEqual(split_sentence_to_array('你好 are  u 可以', True), ['你好', 'are', 'u', '可以'])
        self.assertEqual(split_sentence_to_array('你好 are  u 可以'), ['你', '好', 'are', 'u', '可', '以'])
        self.assertEqual(split_sentence_to_array('傑弗里·德維爾阿杜安'), ['傑', '弗', '里', '·', '德', '維', '爾', '阿', '杜', '安'])
        self.assertEqual(split_sentence_to_array("威廉•布萊克•里奇蒙爵士"),
                         ['威', '廉', '•', '布', '萊', '克', '•', '里', '奇', '蒙', '爵', '士'])
        self.assertEqual(split_sentence_to_array("35°"),
                         ['3', '5', '°'])
        self.assertEqual(split_sentence_to_array("尼爾斯•波耳"),
                         ['尼', '爾', '斯', '•', '波', '耳'])
        self.assertEqual(split_sentence_to_array("i'm fine"),
                         ['i', '\'', 'm', 'fine'])

    def test_join_words_array_to_sentence(self):
        self.assertEqual(join_words_to_sentence(['你', 'ok', "嗎"]), "你 ok 嗎")
        self.assertEqual(join_words_to_sentence(['I', 'am', "可以"]), "I am 可以")

    def test_passage_into_chunk(self):
        self.assertEqual(passage_into_chunk("xxxxxxxx\noo\nyyzz\ngggggg\nkkkk\n", 10),
                         ['xxxxxxxx oo ', 'yyzz gggggg ', 'kkkk '])

    def test_is_all_english(self):
        self.assertTrue(is_all_english("1SGD"))
        self.assertFalse(is_all_english("1SG哦"))

    def test_is_contain_number(self):
        self.assertTrue(is_contain_number("1SGD"))
        self.assertFalse(is_contain_number("SG哦"))

    def test_is_contain_english(self):
        self.assertTrue(is_contain_english("1SGD"))
        self.assertFalse(is_contain_english("1哦"))

    def test_is_list_contain_string(self):
        self.assertFalse(is_list_contain_string("a", ['bdc', 'dcd']))
        self.assertTrue(is_list_contain_string("a", ['a', 'dcd']))
        self.assertTrue(is_list_contain_string("a", ['abcd', 'dcd']))

    def test_full2half(self):
        self.assertEqual(full2half("，,"), ",,")

    def test_half2full(self):
        self.assertEqual(half2full("，,"), "，，")

    def test_sliding_windows(self):
        a = [i for i in range(125)]
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 1)[0]]), 125)
        self.assertGreaterEqual(sum([len(i) for i in sliding_windows(a, 25)[0]]), 125)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 50)[0]]), 200)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 100)[0]]), 200)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 125)[0]]), 125)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 200)[0]]), 125)
        a = ["a"] * 125  # avoid adding same list repeatedly
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 1)[0]]), 1)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 50)[0]]), 50)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 125)[0]]), 125)
        self.assertEqual(sum([len(i) for i in sliding_windows(a, 200)[0]]), 125)

    def test_filter_jaccard_similar_text_from_list(self):
        self.assertEqual(len(filter_jaccard_similar_text_from_list(["abcd", "abc"], 1)), 1)
        self.assertEqual(len(filter_jaccard_similar_text_from_list(["ab", "efc"], 1)), 2)

    if __name__ == '__main__':
        unittest.main()



import os
import random
import time
from random import sample, randint, uniform


def set_seed(seed):
    os.environ['PYTHONHASHSEED'] = str(seed)
    random.seed(seed)
    try:
        import numpy
        numpy.random.seed(seed)
    except:
        pass

    try:
        import numpy
        numpy.random.seed(seed)
        import torch
        torch.manual_seed(seed)
        torch.backends.cudnn.benchmark = False
        torch.backends.cudnn.deterministic = True
        torch.cuda.manual_seed(seed)
        if torch.cuda.is_available() > 0:
            torch.cuda.manual_seed_all(seed)
    except:
        pass


def _checkInt(l):
    return all(isinstance(i, int) for i in l)


def _checkStr(l):
    return all(isinstance(i, str) for i in l)


def _checkFloat(l):
    return any(isinstance(i, float) for i in l)


def random_string(length):
    return ''.join(sample('0123456789ABCDEF', length))


def random_string_with_timestamp(length):
    return str(int(time.time())) + random_string(length)


def random_value_in_array_form(input_array):
    if _checkInt(input_array):
        return randint(input_array[0], input_array[1])
    elif _checkFloat(input_array):
        return uniform(input_array[0], input_array[1])
    elif _checkStr(input_array):
        return input_array[randint(0, len(input_array) - 1)]



from setuptools import setup, find_packages

setup(
    name='nlp2',
    version='1.8.47',
    description='Tool for NLP - handle file and text',
    long_description="Github : https://github.com/voidful/nlp2",
    url='https://github.com/voidful/nlp2',
    author='Voidful',
    author_email='voidful.stack@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='nlp file io string text mining',
    packages=find_packages()
)



from .text import *


def doc2vec_aver(pretrained_emb, emb_size, context):
    docvec = [0] * emb_size
    count = len(context)
    for c in context:
        try:
            docvec = map(sum, zip(docvec, pretrained_emb[c]))
            docvec = map(lambda x: x / count, pretrained_emb[c])
        except Exception as e:
            pass
    return list(docvec)


def doc2vec_max(pretrained_emb, emb_size, context):
    arr_list = []
    for c in context:
        try:
            arr_list.append(pretrained_emb[c])
        except Exception as e:
            pass
    docvec = [max(row) for row in arr_list]
    return list(docvec)


def doc2vec_concat(pretrained_emb, emb_size, context):
    docvec = [0] * emb_size
    docvec = map(sum, zip(docvec, doc2vec_aver(pretrained_emb, emb_size, context)))
    docvec = map(sum, zip(docvec, doc2vec_max(pretrained_emb, emb_size, context)))
    return list(docvec)


def doc2vec_hier(pretrained_emb, emb_size, context, windows):
    arr_list = []
    for i in sliding_windows(context, windows):
        try:
            arr_list.append(doc2vec_aver(pretrained_emb, emb_size, i))
        except Exception as e:
            pass
    docvec = [max(row) for row in arr_list]
    return list(docvec)


def dot(A, B):
    return sum(a * b for a, b in zip(A, B))


def cosine_similarity(vector1, vector2):
    if vector1 == 0 or vector2 == 0:
        return 0
    return dot(vector1, vector2) / ((dot(vector1, vector1) ** .5) * (dot(vector2, vector2) ** .5))
